﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HeuristicLab.DataPreprocessing {
  class SearchLogic : ISearchLogic {
    private readonly IPreprocessingData preprocessingData;

    public SearchLogic(IPreprocessingData thePreprocessingData) {
      preprocessingData = thePreprocessingData;
    }

    public IDictionary<string, IEnumerable<int>> GetMissingValueIndices() {
      var dic = new Dictionary<string, IEnumerable<int>>();
      foreach (string variableName in preprocessingData.VariableNames) {
        dic.Add(variableName, GetMissingValueIndices(preprocessingData.GetColumnIndex(variableName)));
      }
      return dic;
    }

    public bool IsMissingValue(int columnIndex, int rowIndex) {
      if (preprocessingData.IsType<double>(columnIndex)) {
        return double.IsNaN(preprocessingData.GetCell<double>(columnIndex, rowIndex));
      } else if (preprocessingData.IsType<string>(columnIndex)) {
        return string.IsNullOrEmpty(preprocessingData.GetCell<string>(columnIndex, rowIndex));
      } else if (preprocessingData.IsType<DateTime>(columnIndex)) {
        return preprocessingData.GetCell<DateTime>(columnIndex, rowIndex).Equals(DateTime.MinValue);
      } else {
        throw new ArgumentException("cell in column " + columnIndex + " and row index " + rowIndex + " contains a non supported type.");
      }
    }

    public IEnumerable<int> GetMissingValueIndices(int columnIndex) {
      if (preprocessingData.IsType<double>(columnIndex)) {
        return preprocessingData.GetValues<double>(columnIndex).Select((s, i) => new { i, s }).Where(t => double.IsNaN(t.s)).Select(t => t.i);
      } else if (preprocessingData.IsType<string>(columnIndex)) {
        return preprocessingData.GetValues<string>(columnIndex).Select((s, i) => new { i, s }).Where(t => string.IsNullOrEmpty(t.s)).Select(t => t.i);
      } else if (preprocessingData.IsType<DateTime>(columnIndex)) {
        return preprocessingData.GetValues<DateTime>(columnIndex).Select((s, i) => new { i, s }).Where(t => t.s.Equals(DateTime.MinValue)).Select(t => t.i);
      } else {
        throw new ArgumentException("column " + columnIndex + " contains a non supported type.");
      }
    }

  }
}
