﻿using System;

namespace HeuristicLab.DataPreprocessing {
  public interface IStatisticsLogic {

    int GetColumnCount();
    int GetRowCount();
    int GetNumericColumnCount();
    int GetNominalColumnCount();
    int GetMissingValueCount();
    int GetMissingValueCount(int columnIndex);
    int GetRowMissingValueCount(int rowIndex);

    T GetMin<T>(int columnIndex) where T : IComparable<T>;
    T GetMax<T>(int columnIndex) where T : IComparable<T>;
    
    double GetMedian(int columnIndex);
    double GetAverage(int columnIndex);
    DateTime GetMedianDateTime(int columnIndex);
    DateTime GetAverageDateTime(int columnIndex);

    double GetStandardDeviation(int columnIndex);
    double GetVariance(int columnIndex);
    T GetMostCommonValue<T>(int columnIndex);
    int GetDifferentValuesCount<T>(int columnIndex);

    bool IsType<T>(int columnIndex);
    string GetColumnTypeAsString(int columnIndex);

  }
}
