﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.DataPreprocessing {

  [View("Statistics View")]
  [Content(typeof(StatisticsContent), false)]
  public partial class StatisticsView : ItemView {

    private List<List<string>> columnsRowsMatrix;
    private readonly int COLUMNS = 10;

    public new StatisticsContent Content {
      get { return (StatisticsContent)base.Content; }
      set { base.Content = value; }
    }

    public StatisticsView() {
      InitializeComponent();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        txtRows.Text = "";
        txtColumns.Text = "";
        txtNumericColumns.Text = "";
        txtNominalColumns.Text = "";
        txtMissingValuesTotal.Text = "";
        dataGridView.Columns.Clear();
      } else {
        UpdateData();
      }
    }
    private void UpdateData() {
      var logic = Content.StatisticsLogic;
      txtRows.Text = logic.GetRowCount().ToString();
      txtColumns.Text = logic.GetColumnCount().ToString();
      txtNumericColumns.Text = logic.GetNumericColumnCount().ToString();
      txtNominalColumns.Text = logic.GetNominalColumnCount().ToString();
      txtMissingValuesTotal.Text = logic.GetMissingValueCount().ToString();

      columnsRowsMatrix = new List<List<string>>();
      DataGridViewColumn[] columns = new DataGridViewColumn[10];
      for (int i = 0; i < COLUMNS; ++i) {
        var column = new DataGridViewTextBoxColumn();
        column.FillWeight = 1;
        columns[i] = column;
      }

      for (int i = 0; i < logic.GetColumnCount(); ++i) {
        List<string> list;
        if (logic.IsType<double>(i)) {
          list = GetDoubleColumns(i);
        } else if (logic.IsType<string>(i)) {
          list = GetStringColumns(i);
        } else if (logic.IsType<DateTime>(i)) {
          list = GetDateTimeColumns(i);
        } else {
          list = new List<string>();
          for (int j = 0; j < COLUMNS; ++j) {
            list.Add("unknown column type");
          }
        }
        columnsRowsMatrix.Add(list);
      }

      dataGridView.Columns.Clear();
      dataGridView.Columns.AddRange(columns);
      dataGridView.RowCount = columnsRowsMatrix[0].Count;

    }

    private List<string> GetDoubleColumns(int columnIndex) {
      var logic = Content.StatisticsLogic;
      return new List<string> {
        logic.GetColumnTypeAsString(columnIndex),
        logic.GetMissingValueCount(columnIndex).ToString(),
        logic.GetMin<double>(columnIndex).ToString(),
        logic.GetMax<double>(columnIndex).ToString(),
        logic.GetMedian(columnIndex).ToString(),
        logic.GetAverage(columnIndex).ToString(),
        logic.GetStandardDeviation(columnIndex).ToString(),
        logic.GetVariance(columnIndex).ToString(),
        logic.GetMostCommonValue<double>(columnIndex).ToString(),
        logic.GetDifferentValuesCount<double>(columnIndex).ToString()
      };
    }

    private List<string> GetStringColumns(int columnIndex) {
      var logic = Content.StatisticsLogic;
      return new List<string> {
        logic.GetColumnTypeAsString(columnIndex),
        logic.GetMissingValueCount(columnIndex).ToString(),
        "", //min
        "", //max
        "", //median
        "", //average
        "", //standard deviation
        "", //variance
        logic.GetMostCommonValue<string>(columnIndex).ToString(),
        logic.GetDifferentValuesCount<string>(columnIndex).ToString()
      };
    }

    private List<string> GetDateTimeColumns(int columnIndex) {
      var logic = Content.StatisticsLogic;
      return new List<string> {
        logic.GetColumnTypeAsString(columnIndex),
        logic.GetMissingValueCount(columnIndex).ToString(),
        logic.GetMin<DateTime>(columnIndex).ToString(),
        logic.GetMax<DateTime>(columnIndex).ToString(),
        "", //median
        "", //average
        "", //standard deviation
        "", //variance
        logic.GetMostCommonValue<string>(columnIndex).ToString(),
        logic.GetDifferentValuesCount<string>(columnIndex).ToString()
      };
    }

    private List<string> GetDoubleColums(int i) {
      throw new NotImplementedException();
    }

    private void dataGridView_CellValueNeeded(object sender, DataGridViewCellValueEventArgs e) {
      if (Content != null && e.RowIndex < columnsRowsMatrix.Count && e.ColumnIndex < Content.StatisticsLogic.GetColumnCount()) {
        dataGridView[e.ColumnIndex, e.RowIndex].Value = columnsRowsMatrix[e.ColumnIndex][e.RowIndex];
      }
    }
  }
}
